( 3iPort.fs - words to read/write to ports in binary )
\ Assad Ebrahim
\ 2020-09-09---Wed---01:31 - started testing the ICs in Forth
\ 2020-09-12---Sat---23:34 - finished virtual port emulation
\ 2020-10-03---Sat---21:15 - extended to use port4 commands added to 3iForth.avr and in 3iForth.fs

\ Arduino Nano with 3iForth.avr
\ Hardware 1: Built 4-port INPUT to 74154 IC (4-bits to 16-line decoder)
\    G1 control signal on chip pulled low permanently
\    2345 on Nano connecting to ABCD on chip (A=lsb, D = msb), 
\    6 is control pin connected to G2 on chip, pull low to start the decoding
\   LEDs connected via 860R resistor from VCC (5V) to address pin of chip (0-12) so they light when the chip lines are pulled low
\ Hardware 2: Built 4-port OUTPUT from 2x cascaded 74148 ICs and a 7408 (AND) IC into a 16-line to 4-bits priority encoder

\ Usage: help-port
\ The below code writes an address/channel (0-15) coded as 4-bits to a 4-bit port and uses 74154 IC to decode and turn on LED with associated address.  The use of the chip saves consumption of 16 I/O pins instead requiring 5, 4 for 4-bit port, 1 for control line

\ INTEGRATION WITH PORT LEVEL INSTRUCTIONS FROM AVR


: out4  5 4 3 2 ;    \ pins for output port, lsb top of stack (2)
: in4   11 10 9 8 ;  \ pins for input port, lsb on top of stack (8)
6  constant CTRL-OUT \ control signal, active low
12 constant CTRL-IN

\ WRITE / OUTPUT 
: light_ ( -- ) ctrl-out lo ;  \ flow to lights.  decode-start: all commands are now real-time reflected by the IC
: light| ( -- ) ctrl-out hi ;  \ stop lights.  decode-stop: all commands are now buffered, i.e. the uC reflects the pins but the IC does not ref
: _light_ ( hex -- ) port4write light_ ;
: port-test2  16 0 do i dup . _light_ 250 ms key loop light| ; 
: use-port-out  ( m c b a  -- )  \ m = msb
    port4out
    ctrl-out out           \ ctrl line for port
    light| ;                \ disable port (active low)


\ READ / INPUT

\ Hardware: 
\ Arduino Nano with 3iForth.avr
\ connected to 2x 74148 ICs cascaded with 7408 (AND), with pins 9,10,11,12 as data pins (lsb-msb) and pin 8 as enable (pull low)

\ 11 10 9 8 port4in
\ port4read not4 .  \ active low
\ 12 in   \ data valid line for port, active low
: poll-port2 ( -- ) begin port4read not4 12 pinread not if . cr else ." ." drop then key? until ." Done!" ;

: key2light  \ logic is right, but it's laggy and needs key debouncing
    begin port4read not4 
    12 pinread not if port4write 6 lo else drop 6 hi then 
    key? until ." Done!" ;

: read4 
cr
decimal
11 pinread .
10 pinread .
9  pinread .
8  pinread .
;

: read4low
cr
decimal
11 pinread not .
10 pinread not .
9  pinread not .
8  pinread not .
;

: port4readlow port4read not4 . ;


\  THIS IS RUNNING PIN LEVEL READING FROM THE HOST INSTEAD OF USING PORT LEVEL COMMANDS AT THE TARGET LEVEL.
\ Usage: 
\ port4-in   setup ports on Arduino
\ port4read-test   continuously reads the 4-bit input port.  Test by setting 0-7 active low on 74148#1 then 8-15 low on 74148#2
\ Ctrl-C to exit the infinite loop

: alow ( 0/1 -- 1/0 ) 1 + 2 mod ;  \ active low
\ for each individual 74148 IC (8 line to 3 bit priority encoder)
: port3A 10 11 12 9 ;
: port3B 15 16 17 14 ;
: port3-in ( port -- ) in in in in ;
: 3drop 2drop drop ;
: read3 ( port -- lsb0 1 msb2 ) pinread alow if rot pinread alow -rot swap pinread alow swap pinread alow else 3drop then ;
: read3A port3A read3 ;
: read3B port3B read3 ;
: 3.  . . . ; 

\ for cascaded 2x 74148 ICs into 7408 AND chip (16 line to 4 bit priority encoder)
: port4 ( -- lsb ... msb valid ) 8 9 10 11 12 ;  \ pin12 = data valid pin (valid if active low), pins 8-11 are 4-bit port read
: port4-in ( -- ) port4 in in in in in ;  \ WARNING: have port4in in forth.fs
: [read-port4] ( msb ... lsb -- )  4 0 do pinread alow . loop ;
: read4 ( port -- ) pinread alow if  [read-port4] else 4drop then ;

: port4-read  cr begin port4 pinread alow if [read-port4] cr else 4 drop then 0 until ;  \ ctrl-c to exit




: help-port
   cr cr
   cr ." === HELP-PORT ==="
   cr ." Dependencies: 3iForth.avr, 3iForth.fs, 3iLight.fs, 3iPort.fs"
   cr 
   cr ." 4-bit port IN using 7408/2x cascaded 74148 (priority 16 line encoder)"
   cr ." Button inputs, active low outputs from board / inputs to uC"
   cr ."   connect pins 8,9,10,11, pin 8 as low-bit, pin 11 as high-bit, and pin 12 as data valid"
   cr ."   PORT4-IN"
   cr
   cr ." 4-bit port OUT using 74154 IC (4-bit to 16 line decoder)"
   cr ." Active high inputs, active low outputs."
   cr ."    out4 USE-PORT-OUT - activate pins 5 4 3 2 as 4-bit output to 74158 decoder, pin 6 as ctrl-signal"
   cr ."    port-test2 - loop through all 16 lights"
   cr ."    light_  _light_  light|  - enable/turn on light n/disable decode logic (lights showing)"
   cr ."    $f _light_" 
   cr
;

: help-port-pin-grain 
   cr ." out4-set - set pins 2,3,4,5,6 as 4-port OUT and control pin"
   cr ." n >>p  - send number to port"
   cr ." p>   - port off"
   cr ." >p   - port on"
   cr ." port-test    keys-test    - demonstrate functionality"
   cr
   cr ." 4-bit port IN using 2x 74148 IC cascaded into 7408 AND chip (16-line to 4-bit priority encoder)"
   cr ." Active low inputs and active low outputs."
   cr ."     port4-in - set pins 8,9,10,11,12 as 4-port IN and data valid pin"
   cr ."     port4read-test - reads port printing encoded inputs until Ctrl-C pressed"
   cr
;

help-port
